package org.fuseri.moduleexercise.answer;

import org.fuseri.model.dto.exercise.AnswerCreateDto;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.AnswerInQuestionCreateDto;
import org.fuseri.model.dto.exercise.AnswersCreateDto;
import org.fuseri.model.dto.exercise.QuestionUpdateDto;
import org.fuseri.moduleexercise.exercise.Exercise;
import org.fuseri.moduleexercise.question.Question;
import org.fuseri.moduleexercise.question.QuestionService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.HashSet;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SpringBootTest
public class AnswerFacadeTest {

    private final Exercise exercise = new Exercise();
    private final Question question = new Question("text",new HashSet<>(),exercise);


    private final AnswerDto answerDto = new AnswerDto("name",true);

    private final AnswerCreateDto answerCreateDto = new AnswerCreateDto("name",true,1L);
    private final AnswerInQuestionCreateDto inQuestionCreateDto = new AnswerInQuestionCreateDto("name",true);
    private final AnswersCreateDto answersCreateDto = new AnswersCreateDto(1L, List.of(inQuestionCreateDto));

    private final QuestionUpdateDto questionUpdateDto = QuestionUpdateDto.builder().build();

    private final Answer answer = new Answer("name",true,question);

    @MockBean
    AnswerRepository repository;
    @Autowired
    AnswerFacade facade;

    @MockBean
    AnswerService service;

    @MockBean
    QuestionService questionService;

    @MockBean
    AnswerMapper mapper;
    @Test
    void create() {
        long id = 1;
        when(service.find(id)).thenReturn(answer);
        when(mapper.fromCreateDto(inQuestionCreateDto)).thenReturn(answer);
        when(service.create(answer)).thenReturn(answer);
        when(mapper.toDto(answer)).thenReturn(answerDto);
        when(questionService.find(id)).thenReturn(question);
        when(mapper.toDtoList(List.of(answer))).thenReturn(List.of(answerDto));

        var actualDto = facade.createMultiple(answersCreateDto);

        assertEquals(List.of(answerDto), actualDto);

    }

    @Test
    void update() {
        Long id = 1L;
        when(repository.existsById(id)).thenReturn(true);
        when(service.find(id)).thenReturn(answer);
        when(mapper.fromCreateDto(answerCreateDto)).thenReturn(answer);
        when(service.update(answer)).thenReturn(answer);
        when(mapper.toDto(answer)).thenReturn(answerDto);
        AnswerDto actualDto = facade.update(id,answerCreateDto);


        assertEquals(answerDto, actualDto);
    }
    @Test
    void testDelete() {
        Long id = 1L;
        when(service.find(id)).thenReturn(answer);
        when(questionService.find(answer.getId())).thenReturn(question);
        facade.delete(id);
        verify(service).delete(id);
    }

}
