package org.fuseri.moduleexercise.answer;

import org.fuseri.moduleexercise.exercise.Exercise;
import org.fuseri.moduleexercise.exercise.ExerciseRepository;
import org.fuseri.moduleexercise.question.Question;
import org.fuseri.moduleexercise.question.QuestionRepository;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;

@DataJpaTest
public class AnswerRepositoryTest {

    @Autowired
    AnswerRepository repository;

    @Autowired
    QuestionRepository questionRepository;

    @Autowired
    ExerciseRepository exerciseRepository;

    @Autowired
    TestEntityManager entityManager;


//    Question question = new Question("name","desc",2,1L,new HashSet<>());

    Exercise exercise = new Exercise("name","desc",2,1L,new HashSet<>());

    Question question = new Question("text",new HashSet<>(),exercise);

    Answer answer = new Answer("text",false,question);
    Answer answer2 = new Answer("text2",true,question);

    @BeforeEach
    void init() {
        exerciseRepository.save(exercise);
        questionRepository.save(question);
    }

    @Test
    void saveQuestion() {
        Question saved = questionRepository.save(question);

        Assertions.assertNotNull(saved);
        Assertions.assertEquals(question, saved);
    }

    @Test
    void findById() {
        entityManager.persist(answer);
        entityManager.flush();

        Answer found = repository.findById(answer.getId()).orElse(null);
//        Question found = questionRepository.findById(question.getId()).orElse(null);

        Assertions.assertNotNull(found);
        Assertions.assertEquals(found, answer);
    }


    @Test
    void testFindAllQuestions() {
        entityManager.persist(answer);
        entityManager.persist(answer2);

        Page<Answer> coursePage = repository.findAll(PageRequest.of(0, 42));

        Assertions.assertEquals(2, coursePage.getTotalElements());
        Assertions.assertEquals(coursePage.getContent(), Arrays.asList(answer, answer2));
    }


    @Test
    void testFindAllQuestionsEmpty() {
        Page<Answer> coursePage = repository.findAll(PageRequest.of(0, 42));

        Assertions.assertEquals(0, coursePage.getTotalElements());
        Assertions.assertEquals(coursePage.getContent(), new ArrayList<>());
    }

    @Test
    void testDeleteQuestion() {
        Long id = entityManager.persist(answer).getId();
        entityManager.flush();

        questionRepository.deleteById(id);

        Assertions.assertTrue(questionRepository.findById(id).isEmpty());
    }
}
