package org.fuseri.moduleexercise.exercise;

import jakarta.persistence.EntityNotFoundException;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
//import org.fuseri.modulelanguageschool.common.ResourceNotFoundException;

import org.fuseri.model.dto.exercise.ExerciseDto;
import org.fuseri.moduleexercise.question.Question;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.util.Collections;
import java.util.HashSet;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;


@SpringBootTest
public class ExerciseServiceTest {

    @MockBean
    ExerciseRepository repository;

    @Autowired
    ExerciseService service;

    Exercise exercise;
    ExerciseCreateDto exercise2;
    ExerciseCreateDto exercise3;

    @BeforeEach
    void setup() {
        exercise = new Exercise("idioms", "exercise on basic idioms",2,1L,new HashSet<Question>());
        exercise2 = new ExerciseCreateDto("idioms1", "exercise on intermediate idioms", 2, 0);
        exercise3 = new ExerciseCreateDto("idioms2", "exercise on basic idioms", 1, 0L);
    }


    @Test
    void create() {
        when(repository.save(exercise)).thenReturn(exercise);
        Exercise result = service.create(exercise);
        Assertions.assertEquals(exercise, result);
        verify(repository).save(exercise);
    }

    @Test
    void notFound() {
        when(repository.findById(anyLong())).thenReturn(Optional.empty());

        Assertions.assertThrows(EntityNotFoundException.class, () -> service.find(anyLong()));
    }

    @Test
    void find() {
        when(repository.findById(anyLong())).thenReturn(Optional.of(exercise));

        Exercise result = service.find(anyLong());

        Assertions.assertEquals(exercise, result);
        verify(repository).findById(anyLong());
    }

    @Test
    void findAll() {
        Pageable pageable = PageRequest.of(0, 10);
        Page<Exercise> page = new PageImpl<>(Collections.emptyList(), pageable, 0);

        when(repository.findAll(pageable)).thenReturn(page);
        Page<Exercise> result = service.findAll(0);

        Assertions.assertEquals(page, result);
        verify(repository).findAll(pageable);
    }

    @Test
    void findPerDiffPerCourse() {
        PageRequest pageable = PageRequest.of(0, 10);
        Page<Exercise> page = new PageImpl<>(Collections.emptyList(), pageable, 0);

        when(repository.filterPerDifficultyPerCourse(pageable,1L,2)).thenReturn(page);

        Page<Exercise> result = service.findPerDifficultyPerCourse(0,1L,2);

        Assertions.assertEquals(page, result);
        verify(repository).filterPerDifficultyPerCourse(pageable,1L,2);
    }


    @Test
    void getQuestions() {
        PageRequest pageable = PageRequest.of(0, 10);
        Page<Question> page = new PageImpl<>(Collections.emptyList(), pageable, 0);

        when(repository.getQuestions(pageable,1L)).thenReturn(page);
        when(repository.existsById(1L)).thenReturn(true);


        Page<Question> result = service.getQuestions(1L,0);

        Assertions.assertEquals(page, result);
        verify(repository).getQuestions(pageable,1L);
    }
}
